@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-#  Copyright 2017-2021 Ramil Nugmanov <nougmanoff@protonmail.com>
+#  Copyright 2017-2022 Ramil Nugmanov <nougmanoff@protonmail.com>
 #  This file is part of chython.
 #
 #  chython is free software; you can redistribute it and/or modify
@@ -16,6 +16,7 @@
 #  You should have received a copy of the GNU Lesser General Public License
 #  along with this program; if not, see <https://www.gnu.org/licenses/>.
 #
+from CachedMethods import cached_args_method
 from collections import defaultdict, deque
 from functools import cached_property
 from itertools import combinations
@@ -48,15 +49,6 @@ class Rings:
             return _sssr(self.not_special_connectivity, self.rings_count)
         return ()
 
-    @cached_property
-    def aromatic_rings(self: 'Graph') -> Tuple[Tuple[int, ...], ...]:
-        """
-        Aromatic rings atoms numbers
-        """
-        bonds = self._bonds
-        return tuple(ring for ring in self.sssr if bonds[ring[0]][ring[-1]] == 4
-                     and all(bonds[n][m] == 4 for n, m in zip(ring, ring[1:])))
-
     @cached_property
     def atoms_rings(self) -> Dict[int, Tuple[Tuple[int, ...]]]:
         """
@@ -75,6 +67,17 @@ class Rings:
         """
         return {n: tuple(len(r) for r in rs) for n, rs in self.atoms_rings.items()}
 
+    @cached_args_method
+    def is_ring_bond(self: 'Graph', n: int, m: int, /) -> bool:
+        """
+        Check is bond in any ring.
+        """
+        self.bond(n, m)  # check if bond exists
+        try:
+            return not set(self.atoms_rings[n]).isdisjoint(self.atoms_rings[m])
+        except KeyError:
+            return False
+
     @cached_property
     def ring_atoms(self):
         """
