@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-#  Copyright 2017-2021 Ramil Nugmanov <nougmanoff@protonmail.com>
+#  Copyright 2017-2022 Ramil Nugmanov <nougmanoff@protonmail.com>
 #  This file is part of chython.
 #
 #  chython is free software; you can redistribute it and/or modify
@@ -20,7 +20,6 @@ from CachedMethods import cached_method
 from functools import reduce
 from hashlib import sha512
 from itertools import chain
-from math import ceil
 from operator import or_
 from typing import Dict, Iterable, Iterator, Optional, Tuple
 from zlib import compress, decompress
@@ -119,8 +118,11 @@ class ReactionContainer(StandardizeReaction, Calculate2DReaction, DepictReaction
         copy._ReactionContainer__reactants = tuple(x.copy() for x in self.__reactants)
         copy._ReactionContainer__products = tuple(x.copy() for x in self.__products)
         copy._ReactionContainer__reagents = tuple(x.copy() for x in self.__reagents)
-        copy._ReactionContainer__meta = self.__meta.copy()
         copy._ReactionContainer__name = self.__name
+        if self.__meta is None:
+            copy._ReactionContainer__meta = None
+        else:
+            copy._ReactionContainer__meta = self.__meta.copy()
         copy._arrow = self._arrow
         copy._signs = self._signs
         return copy
@@ -144,16 +146,6 @@ class ReactionContainer(StandardizeReaction, Calculate2DReaction, DepictReaction
             p = MoleculeContainer()
         return r ^ p
 
-    @classmethod
-    def from_cgr(cls, cgr: CGRContainer) -> 'ReactionContainer':
-        """
-        Decompose CGR into reaction
-        """
-        if not isinstance(cgr, CGRContainer):
-            raise TypeError('CGR expected')
-        r, p = ~cgr
-        return ReactionContainer(r, p)
-
     def flush_cache(self):
         self.__dict__.clear()
         for m in self.molecules():
@@ -265,8 +257,9 @@ class ReactionContainer(StandardizeReaction, Calculate2DReaction, DepictReaction
     def __getstate__(self):
         state = {'reactants': self.__reactants, 'products': self.__products, 'reagents': self.__reagents,
                  'meta': self.__meta, 'name': self.__name, 'arrow': self._arrow, 'signs': self._signs}
-        import chython
-        if chython.pickle_cache:
+        from chython import pickle_cache
+
+        if pickle_cache:
             state['cache'] = self.__dict__
         return state
 
