@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-#  Copyright 2019-2021 Ramil Nugmanov <nougmanoff@protonmail.com>
+#  Copyright 2019-2022 Ramil Nugmanov <nougmanoff@protonmail.com>
 #  This file is part of chython.
 #
 #  chython is free software; you can redistribute it and/or modify
@@ -17,11 +17,12 @@
 #  along with this program; if not, see <https://www.gnu.org/licenses/>.
 #
 from importlib.util import find_spec
+from .free_wilson import *
 from .functional_groups import functional_groups
 from .grid import grid_depict
 
 
-__all__ = ['functional_groups', 'grid_depict']
+__all__ = ['functional_groups', 'grid_depict', 'fw_prepare_groups', 'fw_onehot_groups']
 
 
 if find_spec('rdkit'):
